/*********************************************************
 * Copyright (C) 2007 VMware, Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation version 2 and no later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 *********************************************************/

#include "driver-config.h"
#include "compat_module.h"
#include <linux/moduleparam.h>
#include "vm_basic_types.h"
#include "vmxnet3_int.h"
#include <linux/pm.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 39)
static u32
vmxnet3_get_rx_csum(struct net_device *netdev)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	return adapter->rxcsum;
}


static int
vmxnet3_set_rx_csum(struct net_device *netdev, u32 val)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	unsigned long flags;

	if (adapter->rxcsum != val) {
		adapter->rxcsum = val;
		if (netif_running(netdev)) {
			if (val)
				set_flag_le64(
				&adapter->shared->devRead.misc.uptFeatures,
				UPT1_F_RXCSUM);
			else
				reset_flag_le64(
				&adapter->shared->devRead.misc.uptFeatures,
				UPT1_F_RXCSUM);

			spin_lock_irqsave(&adapter->cmd_lock, flags);
			VMXNET3_WRITE_BAR1_REG(adapter, VMXNET3_REG_CMD,
					       VMXNET3_CMD_UPDATE_FEATURE);
			spin_unlock_irqrestore(&adapter->cmd_lock, flags);
		}
	}
	return 0;
}


int vmxnet3_set_tso(struct net_device *dev, u32 data) {
	if (data)
		dev->features |= COMPAT_NETIF_F_TSO;
	else
		dev->features &= ~COMPAT_NETIF_F_TSO;

	return 0;
}

static u32
vmxnet3_get_tx_csum(struct net_device *netdev)
{
        return (netdev->features & NETIF_F_HW_CSUM) != 0;
}

static int
vmxnet3_set_tx_csum(struct net_device *netdev, u32 val)
{
        struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
        if (val) {
                netdev->features |= NETIF_F_HW_CSUM;
        } else {
                netdev->features &= ~ NETIF_F_HW_CSUM;
        }
        vmxnet3_vlan_features(adapter, 0, TRUE);
        return 0;
}

#endif

/* per tq stats maintained by the device */
static const struct vmxnet3_stat_desc
vmxnet3_tq_dev_stats[] = {
	/* description,         offset */
	{ "Tx Queue#",        0 },
	{ "  TSO pkts tx",        offsetof(UPT1_TxStats, TSOPktsTxOK) },
	{ "  TSO bytes tx",       offsetof(UPT1_TxStats, TSOBytesTxOK) },
	{ "  ucast pkts tx",      offsetof(UPT1_TxStats, ucastPktsTxOK) },
	{ "  ucast bytes tx",     offsetof(UPT1_TxStats, ucastBytesTxOK) },
	{ "  mcast pkts tx",      offsetof(UPT1_TxStats, mcastPktsTxOK) },
	{ "  mcast bytes tx",     offsetof(UPT1_TxStats, mcastBytesTxOK) },
	{ "  bcast pkts tx",      offsetof(UPT1_TxStats, bcastPktsTxOK) },
	{ "  bcast bytes tx",     offsetof(UPT1_TxStats, bcastBytesTxOK) },
	{ "  pkts tx err",        offsetof(UPT1_TxStats, pktsTxError) },
	{ "  pkts tx discard",    offsetof(UPT1_TxStats, pktsTxDiscard) },
};

/* per tq stats maintained by the driver */
static const struct vmxnet3_stat_desc
vmxnet3_tq_driver_stats[] = {
	/* description,         offset */
	{ "  drv dropped tx total", offsetof(struct vmxnet3_tq_driver_stats,
					drop_total) },
	{ "    too many frags",  offsetof(struct vmxnet3_tq_driver_stats,
					drop_too_many_frags) },
	{ "    giant hdr",       offsetof(struct vmxnet3_tq_driver_stats,
					drop_oversized_hdr) },
	{ "    hdr err",         offsetof(struct vmxnet3_tq_driver_stats,
					drop_hdr_inspect_err) },
	{ "    tso",             offsetof(struct vmxnet3_tq_driver_stats,
					drop_tso) },
	{ "  ring full",          offsetof(struct vmxnet3_tq_driver_stats,
					tx_ring_full) },
	{ "  pkts linearized",    offsetof(struct vmxnet3_tq_driver_stats,
					linearized) },
	{ "  hdr cloned",         offsetof(struct vmxnet3_tq_driver_stats,
					copy_skb_header) },
	{ "  giant hdr",          offsetof(struct vmxnet3_tq_driver_stats,
					oversized_hdr) },
};

/* per rq stats maintained by the device */
static const struct vmxnet3_stat_desc
vmxnet3_rq_dev_stats[] = {
	{ "Rx Queue#",        0 },
	{ "  LRO pkts rx",        offsetof(UPT1_RxStats, LROPktsRxOK) },
	{ "  LRO byte rx",        offsetof(UPT1_RxStats, LROBytesRxOK) },
	{ "  ucast pkts rx",      offsetof(UPT1_RxStats, ucastPktsRxOK) },
	{ "  ucast bytes rx",     offsetof(UPT1_RxStats, ucastBytesRxOK) },
	{ "  mcast pkts rx",      offsetof(UPT1_RxStats, mcastPktsRxOK) },
	{ "  mcast bytes rx",     offsetof(UPT1_RxStats, mcastBytesRxOK) },
	{ "  bcast pkts rx",      offsetof(UPT1_RxStats, bcastPktsRxOK) },
	{ "  bcast bytes rx",     offsetof(UPT1_RxStats, bcastBytesRxOK) },
	{ "  pkts rx out of buf", offsetof(UPT1_RxStats, pktsRxOutOfBuf) },
	{ "  pkts rx err",        offsetof(UPT1_RxStats, pktsRxError) },
};

/* per rq stats maintained by the driver */
static const struct vmxnet3_stat_desc
vmxnet3_rq_driver_stats[] = {
	/* description,         offset */
	{ "  drv dropped rx total", offsetof(struct vmxnet3_rq_driver_stats,
					   drop_total) },
	{ "     err",            offsetof(struct vmxnet3_rq_driver_stats,
					drop_err) },
	{ "     fcs",            offsetof(struct vmxnet3_rq_driver_stats,
					drop_fcs) },
	{ "  rx buf alloc fail", offsetof(struct vmxnet3_rq_driver_stats,
					rx_buf_alloc_failure) },
};

/* gloabl stats maintained by the driver */
static const struct vmxnet3_stat_desc
vmxnet3_global_stats[] = {
	/* description,         offset */
	{ "tx timeout count",   offsetof(struct vmxnet3_adapter,
					 tx_timeout_count) }
};

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)

/* return stats got from adapter/netdev */

struct rtnl_link_stats64 *
vmxnet3_get_stats64(struct net_device *netdev,
		    struct rtnl_link_stats64 *stats)
{
	struct vmxnet3_adapter *adapter;
	struct vmxnet3_tq_driver_stats *drvTxStats;
	struct vmxnet3_rq_driver_stats *drvRxStats;
	struct UPT1_TxStats *devTxStats;
	struct UPT1_RxStats *devRxStats;
	unsigned long flags;
	int i;

	adapter = compat_netdev_priv(netdev);

	/* Collect the dev stats into the shared area */
	spin_lock_irqsave(&adapter->cmd_lock, flags);
	VMXNET3_WRITE_BAR1_REG(adapter, VMXNET3_REG_CMD, VMXNET3_CMD_GET_STATS);
	spin_unlock_irqrestore(&adapter->cmd_lock, flags);

	for (i = 0; i < adapter->num_tx_queues; i++) {
		devTxStats = &adapter->tqd_start[i].stats;
		drvTxStats = &adapter->tx_queue[i].stats;
		stats->tx_packets += devTxStats->ucastPktsTxOK +
					devTxStats->mcastPktsTxOK +
					devTxStats->bcastPktsTxOK;
		stats->tx_bytes += devTxStats->ucastBytesTxOK +
				      devTxStats->mcastBytesTxOK +
				      devTxStats->bcastBytesTxOK;
		stats->tx_errors += devTxStats->pktsTxError;
		stats->tx_dropped += drvTxStats->drop_total;
	}

	for (i = 0; i < adapter->num_rx_queues; i++) {
		devRxStats = &adapter->rqd_start[i].stats;
		drvRxStats = &adapter->rx_queue[i].stats;
		stats->rx_packets += devRxStats->ucastPktsRxOK +
					devRxStats->mcastPktsRxOK +
					devRxStats->bcastPktsRxOK;

		stats->rx_bytes += devRxStats->ucastBytesRxOK +
				      devRxStats->mcastBytesRxOK +
				      devRxStats->bcastBytesRxOK;

		stats->rx_errors += devRxStats->pktsRxError;
		stats->rx_dropped += drvRxStats->drop_total;
		stats->multicast +=  devRxStats->mcastPktsRxOK;
	}
	return stats;
}

#else

/* Returns pointer to net_device_stats struct in the adapter/netdev */
struct net_device_stats *
vmxnet3_get_stats(struct net_device *netdev)
{
	struct vmxnet3_adapter *adapter;
	struct vmxnet3_tq_driver_stats *drvTxStats;
	struct vmxnet3_rq_driver_stats *drvRxStats;
	struct UPT1_TxStats *devTxStats;
	struct UPT1_RxStats *devRxStats;
	struct net_device_stats *net_stats;
	unsigned long flags;
	int i;

	adapter = compat_netdev_priv(netdev);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
	net_stats = &adapter->net_stats;
#else
	net_stats = &netdev->stats;
#endif

	/* Collect the dev stats into the shared area */
	spin_lock_irqsave(&adapter->cmd_lock, flags);
	VMXNET3_WRITE_BAR1_REG(adapter, VMXNET3_REG_CMD, VMXNET3_CMD_GET_STATS);
	spin_unlock_irqrestore(&adapter->cmd_lock, flags);

	memset(net_stats, 0, sizeof(*net_stats));
	for (i = 0; i < adapter->num_tx_queues; i++) {
		devTxStats = &adapter->tqd_start[i].stats;
		drvTxStats = &adapter->tx_queue[i].stats;
		net_stats->tx_packets += devTxStats->ucastPktsTxOK +
					devTxStats->mcastPktsTxOK +
					devTxStats->bcastPktsTxOK;
		net_stats->tx_bytes += devTxStats->ucastBytesTxOK +
				      devTxStats->mcastBytesTxOK +
				      devTxStats->bcastBytesTxOK;
		net_stats->tx_errors += devTxStats->pktsTxError;
		net_stats->tx_dropped += drvTxStats->drop_total;
	}

	for (i = 0; i < adapter->num_rx_queues; i++) {
		devRxStats = &adapter->rqd_start[i].stats;
		drvRxStats = &adapter->rx_queue[i].stats;
		net_stats->rx_packets += devRxStats->ucastPktsRxOK +
					devRxStats->mcastPktsRxOK +
					devRxStats->bcastPktsRxOK;

		net_stats->rx_bytes += devRxStats->ucastBytesRxOK +
				      devRxStats->mcastBytesRxOK +
				      devRxStats->bcastBytesRxOK;

		net_stats->rx_errors += devRxStats->pktsRxError;
		net_stats->rx_dropped += drvRxStats->drop_total;
		net_stats->multicast +=  devRxStats->mcastPktsRxOK;
	}
	return net_stats;
}

#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)

/* Returns the number of counters we will return in vmxnet3_get_ethtool_stats.
 * Assume each counter is uint64.
 */
static int
vmxnet3_get_stats_count(struct net_device *netdev)
{
	return  ARRAY_SIZE(vmxnet3_tq_dev_stats) +
		ARRAY_SIZE(vmxnet3_tq_driver_stats) +
		ARRAY_SIZE(vmxnet3_rq_dev_stats) +
		ARRAY_SIZE(vmxnet3_rq_driver_stats) +
		ARRAY_SIZE(vmxnet3_global_stats);
}

#else

static int
vmxnet3_get_sset_count(struct net_device *netdev, int sset)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	switch (sset) {
	case ETH_SS_STATS:
		return  (ARRAY_SIZE(vmxnet3_tq_dev_stats) +
			 ARRAY_SIZE(vmxnet3_tq_driver_stats)) *
			adapter->num_tx_queues +
			(ARRAY_SIZE(vmxnet3_rq_dev_stats) +
			 ARRAY_SIZE(vmxnet3_rq_driver_stats)) *
			adapter->num_rx_queues +
			ARRAY_SIZE(vmxnet3_global_stats);
	default:
		return -EOPNOTSUPP;
	}
}

#endif


/* Should be multiple of 4 */
#define NUM_TX_REGS	8
#define NUM_RX_REGS	12

static int
vmxnet3_get_regs_len(struct net_device *netdev)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	return (adapter->num_tx_queues * NUM_TX_REGS * sizeof(u32) +
		adapter->num_rx_queues * NUM_RX_REGS * sizeof(u32));
}


/*
 * *drvinfo is updated
 */
static void
vmxnet3_get_drvinfo(struct net_device *netdev, struct ethtool_drvinfo *drvinfo)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);

	strlcpy(drvinfo->driver, vmxnet3_driver_name, sizeof(drvinfo->driver));

	strlcpy(drvinfo->version, VMXNET3_DRIVER_VERSION_REPORT,
		sizeof(drvinfo->version));

#if COMPAT_LINUX_VERSION_CHECK_LT(3, 1, 0)
	strlcpy(drvinfo->fw_version, "N/A", sizeof(drvinfo->fw_version));
#endif

	strlcpy(drvinfo->bus_info, pci_name(adapter->pdev),
		ETHTOOL_BUSINFO_LEN);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
	drvinfo->n_stats = vmxnet3_get_sset_count(netdev, ETH_SS_STATS);
#else
	drvinfo->n_stats = vmxnet3_get_stats_count(netdev);
#endif
	drvinfo->testinfo_len = 0;
	drvinfo->eedump_len   = 0;
	drvinfo->regdump_len  = vmxnet3_get_regs_len(netdev);
}

/* Returns the description strings for the counters returned by
 * vmxnet3_get_ethtool_stats.
 */
static void
vmxnet3_get_strings(struct net_device *netdev, u32 stringset, u8 *buf)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	if (stringset == ETH_SS_STATS) {
		int i, j;
		for (j = 0; j < adapter->num_tx_queues; j++) {
			for (i = 0; i < ARRAY_SIZE(vmxnet3_tq_dev_stats); i++) {
				memcpy(buf, vmxnet3_tq_dev_stats[i].desc,
				       ETH_GSTRING_LEN);
				buf += ETH_GSTRING_LEN;
			}
			for (i = 0; i < ARRAY_SIZE(vmxnet3_tq_driver_stats); i++) {
				memcpy(buf, vmxnet3_tq_driver_stats[i].desc,
				       ETH_GSTRING_LEN);
				buf += ETH_GSTRING_LEN;
			}
		}

		for (j = 0; j < adapter->num_rx_queues; j++) {
			for (i = 0; i < ARRAY_SIZE(vmxnet3_rq_dev_stats); i++) {
				memcpy(buf, vmxnet3_rq_dev_stats[i].desc,
				       ETH_GSTRING_LEN);
				buf += ETH_GSTRING_LEN;
			}
			for (i = 0; i < ARRAY_SIZE(vmxnet3_rq_driver_stats); i++) {
				memcpy(buf, vmxnet3_rq_driver_stats[i].desc,
				       ETH_GSTRING_LEN);
				buf += ETH_GSTRING_LEN;
			}
		}

		for (i = 0; i < ARRAY_SIZE(vmxnet3_global_stats); i++) {
			memcpy(buf, vmxnet3_global_stats[i].desc,
				ETH_GSTRING_LEN);
			buf += ETH_GSTRING_LEN;
		}
	}
}


#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 23)

#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 39)
int vmxnet3_set_features(struct net_device *netdev, compat_netdev_features_t features) {
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	unsigned long flags;
	compat_netdev_features_t changed = features ^ netdev->features;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)
	if (changed & (NETIF_F_RXCSUM | NETIF_F_LRO | NETIF_F_HW_VLAN_CTAG_RX)) {
#else
	if (changed & (NETIF_F_RXCSUM | NETIF_F_LRO | NETIF_F_HW_VLAN_RX)) {
#endif
		if (features & NETIF_F_RXCSUM) {
			adapter->shared->devRead.misc.uptFeatures |=
				UPT1_F_RXCSUM;
			adapter->rxcsum = TRUE;
		} else {
			adapter->shared->devRead.misc.uptFeatures &=
				~UPT1_F_RXCSUM;
			adapter->rxcsum = FALSE;
		}

		if (features & NETIF_F_LRO) {
			adapter->shared->devRead.misc.uptFeatures |=
				UPT1_F_LRO;
			adapter->lro = TRUE;
		} else {
			adapter->shared->devRead.misc.uptFeatures &=
				~UPT1_F_LRO;
			adapter->lro = FALSE;
		}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)
		if (features & NETIF_F_HW_VLAN_CTAG_RX)
#else
		if (features & NETIF_F_HW_VLAN_RX)
#endif
			adapter->shared->devRead.misc.uptFeatures |=
				UPT1_F_RXVLAN;
		else
			adapter->shared->devRead.misc.uptFeatures &=
				~UPT1_F_RXVLAN;

		spin_lock_irqsave(&adapter->cmd_lock, flags);
		VMXNET3_WRITE_BAR1_REG(adapter, VMXNET3_REG_CMD,
				VMXNET3_CMD_UPDATE_FEATURE);
		spin_unlock_irqrestore(&adapter->cmd_lock, flags);
	}
	return 0;
}

#  else

static u32
vmxnet3_get_flags(struct net_device *netdev) {
	return netdev->features;
}

static int
vmxnet3_set_flags(struct net_device *netdev, u32 data) {
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	u8 lro_requested = (data & ETH_FLAG_LRO) == 0 ? 0 : 1;
	u8 lro_present = (netdev->features & NETIF_F_LRO) == 0 ? 0 : 1;
	unsigned long flags;

	if (lro_requested != lro_present) {
		/* toggle the LRO feature*/
		netdev->features ^= NETIF_F_LRO;
		adapter->lro = lro_requested;

		/* update harware LRO capability accordingly */
		if (lro_requested)
			adapter->shared->devRead.misc.uptFeatures |= UPT1_F_LRO;
		else
			adapter->shared->devRead.misc.uptFeatures &=
								~UPT1_F_LRO;
		spin_lock_irqsave(&adapter->cmd_lock, flags);
		VMXNET3_WRITE_BAR1_REG(adapter, VMXNET3_REG_CMD,
				       VMXNET3_CMD_UPDATE_FEATURE);
		spin_unlock_irqrestore(&adapter->cmd_lock, flags);
	}
	return 0;
}
#  endif
#endif


static void
vmxnet3_get_ethtool_stats(struct net_device *netdev,
			  struct ethtool_stats *stats, u64 *buf)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	unsigned long flags;
	u8 *base;
	int i;
	int j = 0;

	spin_lock_irqsave(&adapter->cmd_lock, flags);
	VMXNET3_WRITE_BAR1_REG(adapter, VMXNET3_REG_CMD, VMXNET3_CMD_GET_STATS);
	spin_unlock_irqrestore(&adapter->cmd_lock, flags);

	/* this does assume each counter is 64-bit wide */
	for (j = 0; j < adapter->num_tx_queues; j++) {
		base = (u8 *)&adapter->tqd_start[j].stats;
		*buf++ = (u64)j;
		for (i = 1; i < ARRAY_SIZE(vmxnet3_tq_dev_stats); i++)
			*buf++ = *(u64 *)(base + vmxnet3_tq_dev_stats[i].offset);

		base = (u8 *)&adapter->tx_queue[j].stats;
		for (i = 0; i < ARRAY_SIZE(vmxnet3_tq_driver_stats); i++)
			*buf++ = *(u64 *)(base + vmxnet3_tq_driver_stats[i].offset);
	}

	for (j = 0; j < adapter->num_tx_queues; j++) {
		base = (u8 *)&adapter->rqd_start[j].stats;
		*buf++ = (u64) j;
		for (i = 1; i < ARRAY_SIZE(vmxnet3_rq_dev_stats); i++)
			*buf++ = *(u64 *)(base + vmxnet3_rq_dev_stats[i].offset);

		base = (u8 *)&adapter->rx_queue[j].stats;
		for (i = 0; i < ARRAY_SIZE(vmxnet3_rq_driver_stats); i++)
			*buf++ = *(u64 *)(base + vmxnet3_rq_driver_stats[i].offset);
	}

	base = (u8 *)adapter;
	for (i = 0; i < ARRAY_SIZE(vmxnet3_global_stats); i++)
		*buf++ = *(u64 *)(base + vmxnet3_global_stats[i].offset);
}


static void
vmxnet3_get_regs(struct net_device *netdev, struct ethtool_regs *regs, void *p)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	u32 *buf = p;
	int i = 0, j = 0;

	memset(p, 0, vmxnet3_get_regs_len(netdev));

	regs->version = 1;

	/* Update vmxnet3_get_regs_len if we want to dump more registers */

	/* make each ring use multiple of 16 bytes */
	for (i = 0; i < adapter->num_tx_queues; i++) {
		buf[j++] = adapter->tx_queue[i].tx_ring.next2fill;
		buf[j++] = adapter->tx_queue[i].tx_ring.next2comp;
		buf[j++] = adapter->tx_queue[i].tx_ring.gen;
		buf[j++] = 0;

		buf[j++] = adapter->tx_queue[i].comp_ring.next2proc;
		buf[j++] = adapter->tx_queue[i].comp_ring.gen;
		buf[j++] = adapter->tx_queue[i].stopped;
		buf[j++] = 0;
	}

	for (i = 0; i < adapter->num_rx_queues; i++) {
		buf[j++] = adapter->rx_queue[i].rx_ring[0].next2fill;
		buf[j++] = adapter->rx_queue[i].rx_ring[0].next2comp;
		buf[j++] = adapter->rx_queue[i].rx_ring[0].gen;
		buf[j++] = 0;

		buf[j++] = adapter->rx_queue[i].rx_ring[1].next2fill;
		buf[j++] = adapter->rx_queue[i].rx_ring[1].next2comp;
		buf[j++] = adapter->rx_queue[i].rx_ring[1].gen;
		buf[j++] = 0;

		buf[j++] = adapter->rx_queue[i].comp_ring.next2proc;
		buf[j++] = adapter->rx_queue[i].comp_ring.gen;
		buf[j++] = 0;
		buf[j++] = 0;
	}

}


static void
vmxnet3_get_wol(struct net_device *netdev, struct ethtool_wolinfo *wol)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);

	wol->supported = WAKE_UCAST | WAKE_ARP | WAKE_MAGIC;
	wol->wolopts = adapter->wol;
}


static int
vmxnet3_set_wol(struct net_device *netdev, struct ethtool_wolinfo *wol)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);

	if (wol->wolopts & (WAKE_PHY | WAKE_MCAST | WAKE_BCAST |
			    WAKE_MAGICSECURE)) {
		return -EOPNOTSUPP;
	}

	adapter->wol = wol->wolopts;

	compat_device_set_wakeup_enable(&adapter->pdev->dev, adapter->wol);

	return 0;
}


static int
vmxnet3_get_settings(struct net_device *netdev, struct ethtool_cmd *ecmd)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);

	ecmd->supported = SUPPORTED_10000baseT_Full | SUPPORTED_1000baseT_Full |
			  SUPPORTED_TP;
	ecmd->advertising = ADVERTISED_TP;
	ecmd->port = PORT_TP;
	ecmd->transceiver = XCVR_INTERNAL;

	if (adapter->link_speed) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 39)
		ecmd->speed = adapter->link_speed;
#else
		ethtool_cmd_speed_set(ecmd, adapter->link_speed);
#endif
		ecmd->duplex = DUPLEX_FULL;
	} else {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 39)
		ecmd->speed = -1;
#else
		ethtool_cmd_speed_set(ecmd, -1);
#endif
		ecmd->duplex = -1;
	}
	return 0;
}


static void
vmxnet3_get_ringparam(struct net_device *netdev,
		      struct ethtool_ringparam *param)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);

	param->rx_max_pending = VMXNET3_RX_RING_MAX_SIZE;
	param->tx_max_pending = VMXNET3_TX_RING_MAX_SIZE;
	param->rx_mini_max_pending = 0;
	param->rx_jumbo_max_pending = VMXNET3_RX_RING2_MAX_SIZE;

	param->rx_pending = adapter->rx_queue[0].rx_ring[0].size;
	param->tx_pending = adapter->tx_queue[0].tx_ring.size;
	param->rx_mini_pending = 0;
	param->rx_jumbo_pending = adapter->rx_queue[0].rx_ring[1].size;
}


static int
vmxnet3_set_ringparam(struct net_device *netdev,
		      struct ethtool_ringparam *param)
{
	struct vmxnet3_adapter *adapter = compat_netdev_priv(netdev);
	u32 new_tx_ring_size, new_rx_ring_size, new_rx_ring2_size;
	u32 sz;

	if (param->tx_pending == 0 ||
	    param->tx_pending >	VMXNET3_TX_RING_MAX_SIZE)
		return -EINVAL;

	if (param->rx_pending == 0 ||
	    param->rx_pending >	VMXNET3_RX_RING_MAX_SIZE)
		return -EINVAL;

	if (param->rx_jumbo_pending == 0 ||
            param->rx_jumbo_pending > VMXNET3_RX_RING2_MAX_SIZE)
                return -EINVAL;

	/* if adapter not yet initialized, do nothing */
	if (adapter->rx_buf_per_pkt == 0) {
		printk(KERN_INFO "%s: adapter not completely initialized, "
		       "ring size cannot be changed yet.\n", netdev->name);
		return -EOPNOTSUPP;
	}

	/* round it up to a multiple of VMXNET3_RING_SIZE_ALIGN */
	new_tx_ring_size = (param->tx_pending + VMXNET3_RING_SIZE_MASK) &
		~VMXNET3_RING_SIZE_MASK;
	new_tx_ring_size = min_t(u32, new_tx_ring_size,
				 VMXNET3_TX_RING_MAX_SIZE);
	BUG_ON(new_tx_ring_size > VMXNET3_TX_RING_MAX_SIZE);
	BUG_ON(new_tx_ring_size % VMXNET3_RING_SIZE_ALIGN != 0);

	/* ring0 has to be a multiple of
	 * rx_buf_per_pkt * VMXNET3_RING_SIZE_ALIGN
	 */
	sz = adapter->rx_buf_per_pkt * VMXNET3_RING_SIZE_ALIGN;
	new_rx_ring_size = (param->rx_pending + sz - 1) / sz * sz;
	new_rx_ring_size = min_t(u32, new_rx_ring_size,
			VMXNET3_RX_RING_MAX_SIZE / sz * sz);
	BUG_ON(new_rx_ring_size > VMXNET3_RX_RING_MAX_SIZE);
	BUG_ON(new_rx_ring_size % sz != 0);

	/* ring1 has to be a multiple of VMXNET3_RING_SIZE_ALIGN */
        new_rx_ring2_size = (param->rx_jumbo_pending + VMXNET3_RING_SIZE_MASK) &
                ~VMXNET3_RING_SIZE_MASK;
        new_rx_ring2_size = min_t(u32, new_rx_ring2_size,
                                  VMXNET3_RX_RING2_MAX_SIZE);
        BUG_ON(new_rx_ring2_size > VMXNET3_RX_RING2_MAX_SIZE);
        BUG_ON(new_rx_ring2_size % VMXNET3_RING_SIZE_ALIGN != 0);

	if(adapter->use_adaptive_ring) {
		adapter->use_adaptive_ring = FALSE;
		printk(KERN_INFO "%s: User specified custom ring size, "
		       "disabling adaptive ring support.\n", netdev->name);
	}

	return vmxnet3_set_ringsize(netdev, new_tx_ring_size, new_rx_ring_size,
                                    new_rx_ring2_size);
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
static int
vmxnet3_get_rxnfc(struct net_device *netdev, struct ethtool_rxnfc *info,
#   if COMPAT_LINUX_VERSION_CHECK_LT(3, 2, 0)
                  void *rules)
#   else
		  u32 *rules)
#   endif
{
	struct vmxnet3_adapter *adapter = netdev_priv(netdev);
	switch (info->cmd) {
	case ETHTOOL_GRXRINGS:
		info->data = adapter->num_rx_queues;
		return 0;
	}
	return -EOPNOTSUPP;
}
#endif

#if defined(VMXNET3_RSS) && LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)

#   if COMPAT_LINUX_VERSION_CHECK_LT(3, 3, 0) && !defined(ORACLE_UEK2_300_VMXNET3_FIX)
#   else
static u32
vmxnet3_get_rss_indir_size(struct net_device *netdev)
{
	struct vmxnet3_adapter *adapter = netdev_priv(netdev);
	struct UPT1_RSSConf *rssConf = adapter->rss_conf;

	return rssConf->indTableSize;
}

#   endif

 static int
#   if COMPAT_LINUX_VERSION_CHECK_LT(3, 3, 0) && !defined(ORACLE_UEK2_300_VMXNET3_FIX)
vmxnet3_get_rss_indir(struct net_device *netdev,
		      struct ethtool_rxfh_indir *p)
#   else
vmxnet3_get_rss_indir(struct net_device *netdev, u32 *p)
#   endif
{
	struct vmxnet3_adapter *adapter = netdev_priv(netdev);
	struct UPT1_RSSConf *rssConf = adapter->rss_conf;
#   if COMPAT_LINUX_VERSION_CHECK_LT(3, 3, 0) && !defined(ORACLE_UEK2_300_VMXNET3_FIX)
	unsigned int n = min_t(unsigned int, p->size, rssConf->indTableSize);
	p->size = rssConf->indTableSize;
	while (n--)
		p->ring_index[n] = rssConf->indTable[n];
#   else
	unsigned int n = rssConf->indTableSize;
	BUG_ON(!n);
	while (n--)
		p[n] = rssConf->indTable[n];
#   endif
	return 0;
}

 static int
#   if COMPAT_LINUX_VERSION_CHECK_LT(3, 3, 0) && !defined(ORACLE_UEK2_300_VMXNET3_FIX)
vmxnet3_set_rss_indir(struct net_device *netdev,
		       const struct ethtool_rxfh_indir *p)
#   else
vmxnet3_set_rss_indir(struct net_device *netdev, const u32 *p)
#   endif
 {
        unsigned int i;
        unsigned long flags;
        struct vmxnet3_adapter *adapter = netdev_priv(netdev);
        struct UPT1_RSSConf *rssConf = adapter->rss_conf;
#   if COMPAT_LINUX_VERSION_CHECK_LT(3, 3, 0) && !defined(ORACLE_UEK2_300_VMXNET3_FIX)
	const u32 *idxArray = p->ring_index;
	if (p->size != rssConf->indTableSize)
		return -EINVAL;
#   else
	const u32 *idxArray = p;
#   endif

	for (i = 0; i < rssConf->indTableSize; i++) {
		/*
		 * Return with error code if any of the queue indices
		 * is out of range
		 */
		if (idxArray[i] >= adapter->num_rx_queues)
			return -EINVAL;
	}

	for (i = 0; i < rssConf->indTableSize; i++)
		rssConf->indTable[i] = idxArray[i];

        spin_lock_irqsave(&adapter->cmd_lock, flags);
        VMXNET3_WRITE_BAR1_REG(adapter, VMXNET3_REG_CMD,
			       VMXNET3_CMD_UPDATE_RSSIDT);
	spin_unlock_irqrestore(&adapter->cmd_lock, flags);

	return 0;

}

#endif



static const struct ethtool_ops vmxnet3_ethtool_ops = {
	.get_settings      = vmxnet3_get_settings,
	.get_drvinfo       = vmxnet3_get_drvinfo,
	.get_regs_len      = vmxnet3_get_regs_len,
	.get_regs          = vmxnet3_get_regs,
	.get_wol           = vmxnet3_get_wol,
	.set_wol           = vmxnet3_set_wol,
	.get_link          = ethtool_op_get_link,
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 39)
	.get_rx_csum       = vmxnet3_get_rx_csum,
	.set_rx_csum       = vmxnet3_set_rx_csum,
	.get_tx_csum       = vmxnet3_get_tx_csum,
	.set_tx_csum       = vmxnet3_set_tx_csum,
	.get_sg            = ethtool_op_get_sg,
	.set_sg            = ethtool_op_set_sg,
	.get_tso           = ethtool_op_get_tso,
	.set_tso           = vmxnet3_set_tso,
#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
	.get_flags	   = vmxnet3_get_flags,
	.set_flags	   = vmxnet3_set_flags,
#  endif
#endif
	.get_strings       = vmxnet3_get_strings,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
	.get_sset_count	   = vmxnet3_get_sset_count,
#else
	.get_stats_count   = vmxnet3_get_stats_count,
#endif
	.get_ethtool_stats = vmxnet3_get_ethtool_stats,
	.get_ringparam     = vmxnet3_get_ringparam,
	.set_ringparam     = vmxnet3_set_ringparam,
#if defined(ETHTOOL_GPERMADDR) && (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)) && (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 13))
	.get_perm_addr     = ethtool_op_get_perm_addr,
#endif
#if  LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
	.get_rxnfc         = vmxnet3_get_rxnfc,
#   ifdef VMXNET3_RSS
	.get_rxfh_indir    = vmxnet3_get_rss_indir,
	.set_rxfh_indir    = vmxnet3_set_rss_indir,
#      if COMPAT_LINUX_VERSION_CHECK_LT(3, 3, 0) && !defined(ORACLE_UEK2_300_VMXNET3_FIX)
#      else
	.get_rxfh_indir_size = vmxnet3_get_rss_indir_size,
#      endif
#   endif
#endif
};

void vmxnet3_set_ethtool_ops(struct net_device *netdev)
{
#   if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
	SET_ETHTOOL_OPS(netdev, &vmxnet3_ethtool_ops);
#   else
	/* Need to typecase to avoid the warning for const */
	SET_ETHTOOL_OPS(netdev, (struct ethtool_ops *) &vmxnet3_ethtool_ops);
#   endif
}

